@@ -33,7 +33,6 @@
 #include "gstvaapidecoder_unit.h"
 #include "gstvaapidecoder_priv.h"
 #include "gstvaapidisplay_priv.h"
-#include "gstvaapiobject_priv.h"
 
 #define DEBUG 1
 #include "gstvaapidebug.h"
@@ -234,13 +233,17 @@ ensure_context (GstVaapiDecoderVC1 * decoder)
 
   if (reset_context) {
     GstVaapiContextInfo info;
+    /* *INDENT-OFF* */
+    info = (GstVaapiContextInfo) {
+      .profile = priv->profile,
+      .entrypoint = entrypoint,
+      .chroma_type = GST_VAAPI_CHROMA_TYPE_YUV420,
+      .width = priv->width,
+      .height = priv->height,
+      .ref_frames = 2,
+    };
+    /* *INDENT-ON* */
 
-    info.profile = priv->profile;
-    info.entrypoint = entrypoint;
-    info.chroma_type = GST_VAAPI_CHROMA_TYPE_YUV420;
-    info.width = priv->width;
-    info.height = priv->height;
-    info.ref_frames = 2;
     reset_context =
         gst_vaapi_decoder_ensure_context (GST_VAAPI_DECODER (decoder), &info);
     if (!reset_context)
@@ -466,29 +469,29 @@ get_BFRACTION (guint bfraction)
   }
   bfraction_map[] = {
     {
-    0, GST_VC1_BFRACTION_BASIS / 2}, {
-    1, GST_VC1_BFRACTION_BASIS / 3}, {
-    2, (GST_VC1_BFRACTION_BASIS * 2) / 3}, {
-    3, GST_VC1_BFRACTION_BASIS / 4}, {
-    4, (GST_VC1_BFRACTION_BASIS * 3) / 4}, {
-    5, GST_VC1_BFRACTION_BASIS / 5}, {
-    6, (GST_VC1_BFRACTION_BASIS * 2) / 5}, {
-    7, (GST_VC1_BFRACTION_BASIS * 3) / 5}, {
-    8, (GST_VC1_BFRACTION_BASIS * 4) / 5}, {
-    9, GST_VC1_BFRACTION_BASIS / 6}, {
-    10, (GST_VC1_BFRACTION_BASIS * 5) / 6}, {
-    11, GST_VC1_BFRACTION_BASIS / 7}, {
-    12, (GST_VC1_BFRACTION_BASIS * 2) / 7}, {
-    13, (GST_VC1_BFRACTION_BASIS * 3) / 7}, {
-    14, (GST_VC1_BFRACTION_BASIS * 4) / 7}, {
-    15, (GST_VC1_BFRACTION_BASIS * 5) / 7}, {
-    16, (GST_VC1_BFRACTION_BASIS * 6) / 7}, {
-    17, GST_VC1_BFRACTION_BASIS / 8}, {
-    18, (GST_VC1_BFRACTION_BASIS * 3) / 8}, {
-    19, (GST_VC1_BFRACTION_BASIS * 5) / 8}, {
-    20, (GST_VC1_BFRACTION_BASIS * 7) / 8}, {
-    21, GST_VC1_BFRACTION_RESERVED}, {
-    22, GST_VC1_BFRACTION_PTYPE_BI}
+        0, GST_VC1_BFRACTION_BASIS / 2}, {
+        1, GST_VC1_BFRACTION_BASIS / 3}, {
+        2, (GST_VC1_BFRACTION_BASIS * 2) / 3}, {
+        3, GST_VC1_BFRACTION_BASIS / 4}, {
+        4, (GST_VC1_BFRACTION_BASIS * 3) / 4}, {
+        5, GST_VC1_BFRACTION_BASIS / 5}, {
+        6, (GST_VC1_BFRACTION_BASIS * 2) / 5}, {
+        7, (GST_VC1_BFRACTION_BASIS * 3) / 5}, {
+        8, (GST_VC1_BFRACTION_BASIS * 4) / 5}, {
+        9, GST_VC1_BFRACTION_BASIS / 6}, {
+        10, (GST_VC1_BFRACTION_BASIS * 5) / 6}, {
+        11, GST_VC1_BFRACTION_BASIS / 7}, {
+        12, (GST_VC1_BFRACTION_BASIS * 2) / 7}, {
+        13, (GST_VC1_BFRACTION_BASIS * 3) / 7}, {
+        14, (GST_VC1_BFRACTION_BASIS * 4) / 7}, {
+        15, (GST_VC1_BFRACTION_BASIS * 5) / 7}, {
+        16, (GST_VC1_BFRACTION_BASIS * 6) / 7}, {
+        17, GST_VC1_BFRACTION_BASIS / 8}, {
+        18, (GST_VC1_BFRACTION_BASIS * 3) / 8}, {
+        19, (GST_VC1_BFRACTION_BASIS * 5) / 8}, {
+        20, (GST_VC1_BFRACTION_BASIS * 7) / 8}, {
+        21, GST_VC1_BFRACTION_RESERVED}, {
+        22, GST_VC1_BFRACTION_PTYPE_BI}
   };
 
   if (!bfraction)
@@ -822,9 +825,7 @@ fill_picture (GstVaapiDecoderVC1 * decoder, GstVaapiPicture * picture)
   pic_param->backward_reference_picture = VA_INVALID_ID;
   pic_param->inloop_decoded_picture = VA_INVALID_ID;
   pic_param->sequence_fields.value = 0;
-#if VA_CHECK_VERSION(0,32,0)
   pic_param->sequence_fields.bits.profile = seq_hdr->profile;
-#endif
   pic_param->coded_width = priv->width;
   pic_param->coded_height = priv->height;
   pic_param->entrypoint_fields.value = 0;
