@@ -35,7 +35,6 @@
 #include "gstvaapicontext.h"
 #include "gstvaapiimage.h"
 #include "gstvaapiimage_priv.h"
-#include "gstvaapicontext_overlay.h"
 #include "gstvaapibufferproxy_priv.h"
 
 #define DEBUG 1
@@ -54,7 +53,7 @@ static void
 destroy_subpicture_cb (gpointer subpicture, gpointer surface)
 {
   _gst_vaapi_surface_deassociate_subpicture (surface, subpicture);
-  gst_vaapi_object_unref (subpicture);
+  gst_vaapi_subpicture_unref (subpicture);
 }
 
 static void
@@ -62,23 +61,21 @@ gst_vaapi_surface_destroy_subpictures (GstVaapiSurface * surface)
 {
   if (surface->subpictures) {
     g_ptr_array_foreach (surface->subpictures, destroy_subpicture_cb, surface);
-    g_ptr_array_free (surface->subpictures, TRUE);
-    surface->subpictures = NULL;
+    g_clear_pointer (&surface->subpictures, g_ptr_array_unref);
   }
 }
 
 static void
-gst_vaapi_surface_destroy (GstVaapiSurface * surface)
+gst_vaapi_surface_free (GstVaapiSurface * surface)
 {
-  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  GstVaapiDisplay *const display = GST_VAAPI_SURFACE_DISPLAY (surface);
   VASurfaceID surface_id;
   VAStatus status;
 
-  surface_id = GST_VAAPI_OBJECT_ID (surface);
+  surface_id = GST_VAAPI_SURFACE_ID (surface);
   GST_DEBUG ("surface %" GST_VAAPI_ID_FORMAT, GST_VAAPI_ID_ARGS (surface_id));
 
   gst_vaapi_surface_destroy_subpictures (surface);
-  gst_vaapi_surface_set_parent_context (surface, NULL);
 
   if (surface_id != VA_INVALID_SURFACE) {
     GST_VAAPI_DISPLAY_LOCK (display);
@@ -88,16 +85,19 @@ gst_vaapi_surface_destroy (GstVaapiSurface * surface)
     if (!vaapi_check_status (status, "vaDestroySurfaces()"))
       GST_WARNING ("failed to destroy surface %" GST_VAAPI_ID_FORMAT,
           GST_VAAPI_ID_ARGS (surface_id));
-    GST_VAAPI_OBJECT_ID (surface) = VA_INVALID_SURFACE;
+    GST_VAAPI_SURFACE_ID (surface) = VA_INVALID_SURFACE;
   }
   gst_vaapi_buffer_proxy_replace (&surface->extbuf_proxy, NULL);
+  gst_vaapi_display_replace (&GST_VAAPI_SURFACE_DISPLAY (surface), NULL);
+
+  g_free (surface);
 }
 
 static gboolean
-gst_vaapi_surface_create (GstVaapiSurface * surface,
+gst_vaapi_surface_init (GstVaapiSurface * surface,
     GstVaapiChromaType chroma_type, guint width, guint height)
 {
-  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  GstVaapiDisplay *const display = GST_VAAPI_SURFACE_DISPLAY (surface);
   VASurfaceID surface_id;
   VAStatus status;
   guint va_chroma_format;
@@ -113,13 +113,13 @@ gst_vaapi_surface_create (GstVaapiSurface * surface,
   if (!vaapi_check_status (status, "vaCreateSurfaces()"))
     return FALSE;
 
-  surface->format = GST_VIDEO_FORMAT_UNKNOWN;
-  surface->chroma_type = chroma_type;
-  surface->width = width;
-  surface->height = height;
+  GST_VAAPI_SURFACE_FORMAT (surface) = GST_VIDEO_FORMAT_UNKNOWN;
+  GST_VAAPI_SURFACE_CHROMA_TYPE (surface) = chroma_type;
+  GST_VAAPI_SURFACE_WIDTH (surface) = width;
+  GST_VAAPI_SURFACE_HEIGHT (surface) = height;
 
   GST_DEBUG ("surface %" GST_VAAPI_ID_FORMAT, GST_VAAPI_ID_ARGS (surface_id));
-  GST_VAAPI_OBJECT_ID (surface) = surface_id;
+  GST_VAAPI_SURFACE_ID (surface) = surface_id;
   return TRUE;
 
   /* ERRORS */
@@ -128,18 +128,33 @@ error_unsupported_chroma_type:
   return FALSE;
 }
 
-static gboolean
-gst_vaapi_surface_create_full (GstVaapiSurface * surface,
-    const GstVideoInfo * vip, guint flags)
+static guint
+get_usage_hint (guint alloc_flags)
 {
-  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  guint usage_hints = VA_SURFACE_ATTRIB_USAGE_HINT_GENERIC;
+
+  /* XXX(victor): So far, only media-driver uses hints for encoders
+   * and it doesn't test it as bitwise */
+  if (alloc_flags & GST_VAAPI_SURFACE_ALLOC_FLAG_HINT_DECODER)
+    usage_hints = VA_SURFACE_ATTRIB_USAGE_HINT_DECODER;
+  else if (alloc_flags & GST_VAAPI_SURFACE_ALLOC_FLAG_HINT_ENCODER)
+    usage_hints = VA_SURFACE_ATTRIB_USAGE_HINT_ENCODER;
+
+  return usage_hints;
+}
+
+static gboolean
+gst_vaapi_surface_init_full (GstVaapiSurface * surface,
+    const GstVideoInfo * vip, guint surface_allocation_flags)
+{
+  GstVaapiDisplay *const display = GST_VAAPI_SURFACE_DISPLAY (surface);
   const GstVideoFormat format = GST_VIDEO_INFO_FORMAT (vip);
   VASurfaceID surface_id;
   VAStatus status;
   guint chroma_type, va_chroma_format, i;
   const VAImageFormat *va_format;
-  VASurfaceAttrib attribs[3], *attrib;
-  VASurfaceAttribExternalBuffers extbuf;
+  VASurfaceAttrib attribs[4], *attrib;
+  VASurfaceAttribExternalBuffers extbuf = { 0, };
   gboolean extbuf_needed = FALSE;
 
   va_format = gst_vaapi_video_format_to_va_format (format);
@@ -154,24 +169,23 @@ gst_vaapi_surface_create_full (GstVaapiSurface * surface,
   if (!va_chroma_format)
     goto error_unsupported_format;
 
-  memset (&extbuf, 0, sizeof (extbuf));
   extbuf.pixel_format = va_format->fourcc;
   extbuf.width = GST_VIDEO_INFO_WIDTH (vip);
   extbuf.height = GST_VIDEO_INFO_HEIGHT (vip);
-  if (flags & GST_VAAPI_SURFACE_ALLOC_FLAG_LINEAR_STORAGE) {
+  if (surface_allocation_flags & GST_VAAPI_SURFACE_ALLOC_FLAG_LINEAR_STORAGE) {
     extbuf.flags &= ~VA_SURFACE_EXTBUF_DESC_ENABLE_TILING;
     extbuf_needed = TRUE;
   }
 
   extbuf.num_planes = GST_VIDEO_INFO_N_PLANES (vip);
-  if (flags & GST_VAAPI_SURFACE_ALLOC_FLAG_FIXED_STRIDES) {
-    for (i = 0; i < extbuf.num_planes; i++)
-      extbuf.pitches[i] = GST_VIDEO_INFO_PLANE_STRIDE (vip, i);
-    extbuf_needed = TRUE;
-  }
-  if (flags & GST_VAAPI_SURFACE_ALLOC_FLAG_FIXED_OFFSETS) {
-    for (i = 0; i < extbuf.num_planes; i++)
-      extbuf.offsets[i] = GST_VIDEO_INFO_PLANE_OFFSET (vip, i);
+  if (surface_allocation_flags & (GST_VAAPI_SURFACE_ALLOC_FLAG_FIXED_STRIDES |
+          GST_VAAPI_SURFACE_ALLOC_FLAG_FIXED_OFFSETS)) {
+    for (i = 0; i < extbuf.num_planes; i++) {
+      if (surface_allocation_flags & GST_VAAPI_SURFACE_ALLOC_FLAG_FIXED_STRIDES)
+        extbuf.pitches[i] = GST_VIDEO_INFO_PLANE_STRIDE (vip, i);
+      if (surface_allocation_flags & GST_VAAPI_SURFACE_ALLOC_FLAG_FIXED_OFFSETS)
+        extbuf.offsets[i] = GST_VIDEO_INFO_PLANE_OFFSET (vip, i);
+    }
     extbuf_needed = TRUE;
   }
 
@@ -182,6 +196,12 @@ gst_vaapi_surface_create_full (GstVaapiSurface * surface,
   attrib->value.value.i = va_format->fourcc;
   attrib++;
 
+  attrib->flags = VA_SURFACE_ATTRIB_SETTABLE;
+  attrib->type = VASurfaceAttribUsageHint;
+  attrib->value.type = VAGenericValueTypeInteger;
+  attrib->value.value.i = get_usage_hint (surface_allocation_flags);
+  attrib++;
+
   if (extbuf_needed) {
     attrib->flags = VA_SURFACE_ATTRIB_SETTABLE;
     attrib->type = VASurfaceAttribMemoryType;
@@ -204,13 +224,13 @@ gst_vaapi_surface_create_full (GstVaapiSurface * surface,
   if (!vaapi_check_status (status, "vaCreateSurfaces()"))
     return FALSE;
 
-  surface->format = format;
-  surface->chroma_type = chroma_type;
-  surface->width = extbuf.width;
-  surface->height = extbuf.height;
+  GST_VAAPI_SURFACE_FORMAT (surface) = format;
+  GST_VAAPI_SURFACE_CHROMA_TYPE (surface) = chroma_type;
+  GST_VAAPI_SURFACE_WIDTH (surface) = extbuf.width;
+  GST_VAAPI_SURFACE_HEIGHT (surface) = extbuf.height;
 
   GST_DEBUG ("surface %" GST_VAAPI_ID_FORMAT, GST_VAAPI_ID_ARGS (surface_id));
-  GST_VAAPI_OBJECT_ID (surface) = surface_id;
+  GST_VAAPI_SURFACE_ID (surface) = surface_id;
   return TRUE;
 
   /* ERRORS */
@@ -221,17 +241,17 @@ error_unsupported_format:
 }
 
 static gboolean
-gst_vaapi_surface_create_from_buffer_proxy (GstVaapiSurface * surface,
+gst_vaapi_surface_init_from_buffer_proxy (GstVaapiSurface * surface,
     GstVaapiBufferProxy * proxy, const GstVideoInfo * vip)
 {
-  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  GstVaapiDisplay *const display = GST_VAAPI_SURFACE_DISPLAY (surface);
   GstVideoFormat format;
   VASurfaceID surface_id;
   VAStatus status;
   guint chroma_type, va_chroma_format;
   const VAImageFormat *va_format;
   VASurfaceAttrib attribs[2], *attrib;
-  VASurfaceAttribExternalBuffers extbuf;
+  VASurfaceAttribExternalBuffers extbuf = { 0, };
   unsigned long extbuf_handle;
   guint i, width, height;
 
@@ -289,13 +309,13 @@ gst_vaapi_surface_create_from_buffer_proxy (GstVaapiSurface * surface,
   if (!vaapi_check_status (status, "vaCreateSurfaces()"))
     return FALSE;
 
-  surface->format = format;
-  surface->chroma_type = chroma_type;
-  surface->width = width;
-  surface->height = height;
+  GST_VAAPI_SURFACE_FORMAT (surface) = format;
+  GST_VAAPI_SURFACE_CHROMA_TYPE (surface) = chroma_type;
+  GST_VAAPI_SURFACE_WIDTH (surface) = width;
+  GST_VAAPI_SURFACE_HEIGHT (surface) = height;
 
   GST_DEBUG ("surface %" GST_VAAPI_ID_FORMAT, GST_VAAPI_ID_ARGS (surface_id));
-  GST_VAAPI_OBJECT_ID (surface) = surface_id;
+  GST_VAAPI_SURFACE_ID (surface) = surface_id;
   return TRUE;
 
   /* ERRORS */
@@ -305,52 +325,40 @@ error_unsupported_format:
   return FALSE;
 }
 
-#define gst_vaapi_surface_finalize gst_vaapi_surface_destroy
-GST_VAAPI_OBJECT_DEFINE_CLASS (GstVaapiSurface, gst_vaapi_surface);
+GST_DEFINE_MINI_OBJECT_TYPE (GstVaapiSurface, gst_vaapi_surface);
 
-/**
- * gst_vaapi_surface_new_from_formats:
- * @display: a #GstVaapiDisplay
- * @chroma_type: the surface chroma format
- * @width: the requested surface width
- * @height: the requested surface height
- * @formats: the limited format list
- *
- * Creates a new #GstVaapiSurface with a @chroma_type valid for any
- * format in @formats; If there aren't any, the returned surface is
- * created forcing the passed @chroma_type.
- *
- * Return value: the newly allocated #GstVaapiSurface object
- */
-GstVaapiSurface *
-gst_vaapi_surface_new_from_formats (GstVaapiDisplay * display,
-    GstVaapiChromaType chroma_type, guint width, guint height, GArray * formats)
+static GstVaapiSurface *
+gst_vaapi_surface_create (GstVaapiDisplay * display)
 {
-  GstVaapiSurface *surface;
-  guint i;
-
-  for (i = 0; i < formats->len; i++) {
-    GstVideoFormat format = g_array_index (formats, GstVideoFormat, i);
-    if (format == gst_vaapi_video_format_from_chroma (chroma_type))
-      return gst_vaapi_surface_new (display, chroma_type, width, height);
-  }
-
-  /* Fallback: if there's no format valid for the chroma type let's
-   * just use the passed chroma */
-  surface = gst_vaapi_object_new (gst_vaapi_surface_class (), display);
+  GstVaapiSurface *surface = g_new (GstVaapiSurface, 1);
   if (!surface)
     return NULL;
-  if (!gst_vaapi_surface_create (surface, chroma_type, width, height))
-    goto error;
+
+  gst_mini_object_init (GST_MINI_OBJECT_CAST (surface), 0,
+      GST_TYPE_VAAPI_SURFACE, NULL, NULL,
+      (GstMiniObjectFreeFunction) gst_vaapi_surface_free);
+
+  GST_VAAPI_SURFACE_DISPLAY (surface) = gst_object_ref (display);
+  GST_VAAPI_SURFACE_ID (surface) = VA_INVALID_ID;
+  surface->extbuf_proxy = NULL;
+  surface->subpictures = NULL;
 
   return surface;
+}
 
-  /* ERRORS */
-error:
-  {
-    gst_vaapi_object_unref (surface);
-    return NULL;
-  }
+/**
+ * gst_vaapi_surface_get_display:
+ * @surface: a #GstVaapiSurface
+ *
+ * Returns the #GstVaapiDisplay this @surface is bound to.
+ *
+ * Return value: the parent #GstVaapiDisplay object
+ */
+GstVaapiDisplay *
+gst_vaapi_surface_get_display (GstVaapiSurface * surface)
+{
+  g_return_val_if_fail (surface != NULL, NULL);
+  return GST_VAAPI_SURFACE_DISPLAY (surface);
 }
 
 /**
@@ -363,6 +371,9 @@ error:
  * Creates a new #GstVaapiSurface with the specified chroma format and
  * dimensions.
  *
+ * NOTE: this method for creating surfaces uses deprecated VA API,
+ * which is still used by drivers (v.gr. i965 for jpeg decoding).
+ *
  * Return value: the newly allocated #GstVaapiSurface object
  */
 GstVaapiSurface *
@@ -373,30 +384,18 @@ gst_vaapi_surface_new (GstVaapiDisplay * display,
 
   GST_DEBUG ("size %ux%u, chroma type 0x%x", width, height, chroma_type);
 
-  surface = gst_vaapi_object_new (gst_vaapi_surface_class (), display);
+  surface = gst_vaapi_surface_create (display);
   if (!surface)
     return NULL;
 
-  /* first try a recent version of vaCreateSurface, and later use as
-   * fallback its old version */
-  {
-    GstVideoInfo vi;
-    GstVideoFormat surface_format;
-
-    surface_format = gst_vaapi_video_format_from_chroma (chroma_type);
-    gst_video_info_set_format (&vi, surface_format, width, height);
-
-    if (gst_vaapi_surface_create_full (surface, &vi, 0))
-      return surface;
-  }
-  if (!gst_vaapi_surface_create (surface, chroma_type, width, height))
+  if (!gst_vaapi_surface_init (surface, chroma_type, width, height))
     goto error;
   return surface;
 
   /* ERRORS */
 error:
   {
-    gst_vaapi_object_unref (surface);
+    gst_vaapi_surface_unref (surface);
     return NULL;
   }
 }
@@ -405,7 +404,7 @@ error:
  * gst_vaapi_surface_new_full:
  * @display: a #GstVaapiDisplay
  * @vip: the pointer to a #GstVideoInfo
- * @flags: (optional) allocation flags
+ * @surface_allocation_flags: (optional) allocation flags
  *
  * Creates a new #GstVaapiSurface with the specified video information
  * and optional #GstVaapiSurfaceAllocFlags
@@ -415,27 +414,28 @@ error:
  *   supported or failed.
  */
 GstVaapiSurface *
-gst_vaapi_surface_new_full (GstVaapiDisplay * display,
-    const GstVideoInfo * vip, guint flags)
+gst_vaapi_surface_new_full (GstVaapiDisplay * display, const GstVideoInfo * vip,
+    guint surface_allocation_flags)
 {
   GstVaapiSurface *surface;
 
   GST_DEBUG ("size %ux%u, format %s, flags 0x%08x", GST_VIDEO_INFO_WIDTH (vip),
       GST_VIDEO_INFO_HEIGHT (vip),
-      gst_vaapi_video_format_to_string (GST_VIDEO_INFO_FORMAT (vip)), flags);
+      gst_vaapi_video_format_to_string (GST_VIDEO_INFO_FORMAT (vip)),
+      surface_allocation_flags);
 
-  surface = gst_vaapi_object_new (gst_vaapi_surface_class (), display);
+  surface = gst_vaapi_surface_create (display);
   if (!surface)
     return NULL;
 
-  if (!gst_vaapi_surface_create_full (surface, vip, flags))
+  if (!gst_vaapi_surface_init_full (surface, vip, surface_allocation_flags))
     goto error;
   return surface;
 
   /* ERRORS */
 error:
   {
-    gst_vaapi_object_unref (surface);
+    gst_vaapi_surface_unref (surface);
     return NULL;
   }
 }
@@ -446,6 +446,7 @@ error:
  * @format: the surface format
  * @width: the requested surface width
  * @height: the requested surface height
+ * @surface_allocation_flags: (optional) allocation flags
  *
  * Creates a new #GstVaapiSurface with the specified pixel format and
  * dimensions.
@@ -456,12 +457,13 @@ error:
  */
 GstVaapiSurface *
 gst_vaapi_surface_new_with_format (GstVaapiDisplay * display,
-    GstVideoFormat format, guint width, guint height)
+    GstVideoFormat format, guint width, guint height,
+    guint surface_allocation_flags)
 {
   GstVideoInfo vi;
 
   gst_video_info_set_format (&vi, format, width, height);
-  return gst_vaapi_surface_new_full (display, &vi, 0);
+  return gst_vaapi_surface_new_full (display, &vi, surface_allocation_flags);
 }
 
 /**
@@ -489,19 +491,22 @@ gst_vaapi_surface_new_from_buffer_proxy (GstVaapiDisplay * display,
 
   g_return_val_if_fail (proxy != NULL, NULL);
   g_return_val_if_fail (info != NULL, NULL);
+  g_return_val_if_fail (!proxy->surface, NULL);
 
-  surface = gst_vaapi_object_new (gst_vaapi_surface_class (), display);
+  surface = gst_vaapi_surface_create (display);
   if (!surface)
     return NULL;
 
-  if (!gst_vaapi_surface_create_from_buffer_proxy (surface, proxy, info))
+  if (!gst_vaapi_surface_init_from_buffer_proxy (surface, proxy, info))
     goto error;
+
+  proxy->surface = GST_MINI_OBJECT_CAST (surface);
   return surface;
 
   /* ERRORS */
 error:
   {
-    gst_vaapi_object_unref (surface);
+    gst_vaapi_surface_unref (surface);
     return NULL;
   }
 }
@@ -519,7 +524,7 @@ gst_vaapi_surface_get_id (GstVaapiSurface * surface)
 {
   g_return_val_if_fail (surface != NULL, VA_INVALID_SURFACE);
 
-  return GST_VAAPI_OBJECT_ID (surface);
+  return GST_VAAPI_SURFACE_ID (surface);
 }
 
 /**
@@ -554,14 +559,14 @@ gst_vaapi_surface_get_format (GstVaapiSurface * surface)
   g_return_val_if_fail (surface != NULL, 0);
 
   /* Try to determine the underlying VA surface format */
-  if (surface->format == GST_VIDEO_FORMAT_UNKNOWN) {
+  if (GST_VAAPI_SURFACE_FORMAT (surface) == GST_VIDEO_FORMAT_UNKNOWN) {
     GstVaapiImage *const image = gst_vaapi_surface_derive_image (surface);
     if (image) {
-      surface->format = GST_VAAPI_IMAGE_FORMAT (image);
-      gst_vaapi_object_unref (image);
+      GST_VAAPI_SURFACE_FORMAT (surface) = GST_VAAPI_IMAGE_FORMAT (image);
+      gst_vaapi_image_unref (image);
     }
-    if (surface->format == GST_VIDEO_FORMAT_UNKNOWN)
-      surface->format = GST_VIDEO_FORMAT_ENCODED;
+    if (GST_VAAPI_SURFACE_FORMAT (surface) == GST_VIDEO_FORMAT_UNKNOWN)
+      GST_VAAPI_SURFACE_FORMAT (surface) = GST_VIDEO_FORMAT_ENCODED;
   }
   return GST_VAAPI_SURFACE_FORMAT (surface);
 }
@@ -619,42 +624,6 @@ gst_vaapi_surface_get_size (GstVaapiSurface * surface,
     *height_ptr = GST_VAAPI_SURFACE_HEIGHT (surface);
 }
 
-/**
- * gst_vaapi_surface_set_parent_context:
- * @surface: a #GstVaapiSurface
- * @context: a #GstVaapiContext
- *
- * Sets new parent context, or clears any parent context if @context
- * is %NULL. This function owns an extra reference to the context,
- * which will be released when the surface is destroyed.
- */
-void
-gst_vaapi_surface_set_parent_context (GstVaapiSurface * surface,
-    GstVaapiContext * context)
-{
-  g_return_if_fail (surface != NULL);
-
-  surface->parent_context = NULL;
-}
-
-/**
- * gst_vaapi_surface_get_parent_context:
- * @surface: a #GstVaapiSurface
- *
- * Retrieves the parent #GstVaapiContext, or %NULL if there is
- * none. The surface shall still own a reference to the context.
- * i.e. the caller shall not unreference the returned context object.
- *
- * Return value: the parent context, if any.
- */
-GstVaapiContext *
-gst_vaapi_surface_get_parent_context (GstVaapiSurface * surface)
-{
-  g_return_val_if_fail (surface != NULL, NULL);
-
-  return surface->parent_context;
-}
-
 /**
  * gst_vaapi_surface_derive_image:
  * @surface: a #GstVaapiSurface
@@ -674,7 +643,7 @@ gst_vaapi_surface_get_parent_context (GstVaapiSurface * surface)
  * unreferenced when it's no longer needed. The image and image buffer
  * data structures will be destroyed. However, the surface contents
  * will remain unchanged until destroyed through the last call to
- * gst_vaapi_object_unref().
+ * gst_vaapi_image_unref().
  *
  * Return value: the newly allocated #GstVaapiImage object, or %NULL
  *   on failure
@@ -689,13 +658,13 @@ gst_vaapi_surface_derive_image (GstVaapiSurface * surface)
 
   g_return_val_if_fail (surface != NULL, NULL);
 
-  display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  display = GST_VAAPI_SURFACE_DISPLAY (surface);
   va_image.image_id = VA_INVALID_ID;
   va_image.buf = VA_INVALID_ID;
 
   GST_VAAPI_DISPLAY_LOCK (display);
   status = vaDeriveImage (GST_VAAPI_DISPLAY_VADISPLAY (display),
-      GST_VAAPI_OBJECT_ID (surface), &va_image);
+      GST_VAAPI_SURFACE_ID (surface), &va_image);
   GST_VAAPI_DISPLAY_UNLOCK (display);
   if (!vaapi_check_status (status, "vaDeriveImage()"))
     return NULL;
@@ -729,22 +698,23 @@ gst_vaapi_surface_get_image (GstVaapiSurface * surface, GstVaapiImage * image)
   g_return_val_if_fail (surface != NULL, FALSE);
   g_return_val_if_fail (image != NULL, FALSE);
 
-  display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  display = GST_VAAPI_SURFACE_DISPLAY (surface);
   if (!display)
     return FALSE;
 
   width = GST_VAAPI_IMAGE_WIDTH (image);
   height = GST_VAAPI_IMAGE_HEIGHT (image);
-  if (width != surface->width || height != surface->height)
+  if (width != GST_VAAPI_SURFACE_WIDTH (surface)
+      || height != GST_VAAPI_SURFACE_HEIGHT (surface))
     return FALSE;
 
-  image_id = GST_VAAPI_OBJECT_ID (image);
+  image_id = GST_VAAPI_IMAGE_ID (image);
   if (image_id == VA_INVALID_ID)
     return FALSE;
 
   GST_VAAPI_DISPLAY_LOCK (display);
   status = vaGetImage (GST_VAAPI_DISPLAY_VADISPLAY (display),
-      GST_VAAPI_OBJECT_ID (surface), 0, 0, width, height, image_id);
+      GST_VAAPI_SURFACE_ID (surface), 0, 0, width, height, image_id);
   GST_VAAPI_DISPLAY_UNLOCK (display);
   if (!vaapi_check_status (status, "vaGetImage()"))
     return FALSE;
@@ -773,23 +743,24 @@ gst_vaapi_surface_put_image (GstVaapiSurface * surface, GstVaapiImage * image)
   g_return_val_if_fail (surface != NULL, FALSE);
   g_return_val_if_fail (image != NULL, FALSE);
 
-  display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  display = GST_VAAPI_SURFACE_DISPLAY (surface);
   if (!display)
     return FALSE;
 
   width = GST_VAAPI_IMAGE_WIDTH (image);
   height = GST_VAAPI_IMAGE_HEIGHT (image);
-  if (width != surface->width || height != surface->height)
+  if (width != GST_VAAPI_SURFACE_WIDTH (surface)
+      || height != GST_VAAPI_SURFACE_HEIGHT (surface))
     return FALSE;
 
-  image_id = GST_VAAPI_OBJECT_ID (image);
+  image_id = GST_VAAPI_IMAGE_ID (image);
   if (image_id == VA_INVALID_ID)
     return FALSE;
 
   GST_VAAPI_DISPLAY_LOCK (display);
   status = vaPutImage (GST_VAAPI_DISPLAY_VADISPLAY (display),
-      GST_VAAPI_OBJECT_ID (surface), image_id, 0, 0, width, height,
-      0, 0, width, height);
+      GST_VAAPI_SURFACE_ID (surface), image_id, 0, 0, width, height, 0, 0,
+      width, height);
   GST_VAAPI_DISPLAY_UNLOCK (display);
   if (!vaapi_check_status (status, "vaPutImage()"))
     return FALSE;
@@ -833,7 +804,7 @@ gst_vaapi_surface_associate_subpicture (GstVaapiSurface * surface,
 
   if (g_ptr_array_remove_fast (surface->subpictures, subpicture)) {
     success = _gst_vaapi_surface_deassociate_subpicture (surface, subpicture);
-    gst_vaapi_object_unref (subpicture);
+    gst_vaapi_subpicture_unref (subpicture);
     if (!success)
       return FALSE;
   }
@@ -843,7 +814,8 @@ gst_vaapi_surface_associate_subpicture (GstVaapiSurface * surface,
   if (!success)
     return FALSE;
 
-  g_ptr_array_add (surface->subpictures, gst_vaapi_object_ref (subpicture));
+  g_ptr_array_add (surface->subpictures,
+      gst_mini_object_ref (GST_MINI_OBJECT_CAST (subpicture)));
   return TRUE;
 }
 
@@ -858,11 +830,11 @@ _gst_vaapi_surface_associate_subpicture (GstVaapiSurface * surface,
   VASurfaceID surface_id;
   VAStatus status;
 
-  display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  display = GST_VAAPI_SURFACE_DISPLAY (surface);
   if (!display)
     return FALSE;
 
-  surface_id = GST_VAAPI_OBJECT_ID (surface);
+  surface_id = GST_VAAPI_SURFACE_ID (surface);
   if (surface_id == VA_INVALID_SURFACE)
     return FALSE;
 
@@ -881,13 +853,13 @@ _gst_vaapi_surface_associate_subpicture (GstVaapiSurface * surface,
     dst_rect = &dst_rect_default;
     dst_rect_default.x = 0;
     dst_rect_default.y = 0;
-    dst_rect_default.width = surface->width;
-    dst_rect_default.height = surface->height;
+    dst_rect_default.width = GST_VAAPI_SURFACE_WIDTH (surface);
+    dst_rect_default.height = GST_VAAPI_SURFACE_HEIGHT (surface);
   }
 
   GST_VAAPI_DISPLAY_LOCK (display);
   status = vaAssociateSubpicture (GST_VAAPI_DISPLAY_VADISPLAY (display),
-      GST_VAAPI_OBJECT_ID (subpicture), &surface_id, 1,
+      GST_VAAPI_SUBPICTURE_ID (subpicture), &surface_id, 1,
       src_rect->x, src_rect->y, src_rect->width, src_rect->height,
       dst_rect->x, dst_rect->y, dst_rect->width, dst_rect->height,
       from_GstVaapiSubpictureFlags (gst_vaapi_subpicture_get_flags
@@ -924,13 +896,13 @@ gst_vaapi_surface_deassociate_subpicture (GstVaapiSurface * surface,
   if (!g_ptr_array_remove_fast (surface->subpictures, subpicture)) {
     GST_DEBUG ("subpicture %" GST_VAAPI_ID_FORMAT " was not bound to "
         "surface %" GST_VAAPI_ID_FORMAT,
-        GST_VAAPI_ID_ARGS (GST_VAAPI_OBJECT_ID (subpicture)),
-        GST_VAAPI_ID_ARGS (GST_VAAPI_OBJECT_ID (surface)));
+        GST_VAAPI_ID_ARGS (GST_VAAPI_SUBPICTURE_ID (subpicture)),
+        GST_VAAPI_ID_ARGS (GST_VAAPI_SURFACE_ID (surface)));
     return TRUE;
   }
 
   success = _gst_vaapi_surface_deassociate_subpicture (surface, subpicture);
-  gst_vaapi_object_unref (subpicture);
+  gst_vaapi_subpicture_unref (subpicture);
   return success;
 }
 
@@ -942,17 +914,17 @@ _gst_vaapi_surface_deassociate_subpicture (GstVaapiSurface * surface,
   VASurfaceID surface_id;
   VAStatus status;
 
-  display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  display = GST_VAAPI_SURFACE_DISPLAY (surface);
   if (!display)
     return FALSE;
 
-  surface_id = GST_VAAPI_OBJECT_ID (surface);
+  surface_id = GST_VAAPI_SURFACE_ID (surface);
   if (surface_id == VA_INVALID_SURFACE)
     return FALSE;
 
   GST_VAAPI_DISPLAY_LOCK (display);
   status = vaDeassociateSubpicture (GST_VAAPI_DISPLAY_VADISPLAY (display),
-      GST_VAAPI_OBJECT_ID (subpicture), &surface_id, 1);
+      GST_VAAPI_SUBPICTURE_ID (subpicture), &surface_id, 1);
   GST_VAAPI_DISPLAY_UNLOCK (display);
   if (!vaapi_check_status (status, "vaDeassociateSubpicture()"))
     return FALSE;
@@ -977,13 +949,13 @@ gst_vaapi_surface_sync (GstVaapiSurface * surface)
 
   g_return_val_if_fail (surface != NULL, FALSE);
 
-  display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  display = GST_VAAPI_SURFACE_DISPLAY (surface);
   if (!display)
     return FALSE;
 
   GST_VAAPI_DISPLAY_LOCK (display);
   status = vaSyncSurface (GST_VAAPI_DISPLAY_VADISPLAY (display),
-      GST_VAAPI_OBJECT_ID (surface));
+      GST_VAAPI_SURFACE_ID (surface));
   GST_VAAPI_DISPLAY_UNLOCK (display);
   if (!vaapi_check_status (status, "vaSyncSurface()"))
     return FALSE;
@@ -1005,15 +977,16 @@ gboolean
 gst_vaapi_surface_query_status (GstVaapiSurface * surface,
     GstVaapiSurfaceStatus * pstatus)
 {
+  GstVaapiDisplay *const display = GST_VAAPI_SURFACE_DISPLAY (surface);
   VASurfaceStatus surface_status;
   VAStatus status;
 
   g_return_val_if_fail (surface != NULL, FALSE);
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (surface);
-  status = vaQuerySurfaceStatus (GST_VAAPI_OBJECT_VADISPLAY (surface),
-      GST_VAAPI_OBJECT_ID (surface), &surface_status);
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (surface);
+  GST_VAAPI_DISPLAY_LOCK (display);
+  status = vaQuerySurfaceStatus (GST_VAAPI_DISPLAY_VADISPLAY (display),
+      GST_VAAPI_SURFACE_ID (surface), &surface_status);
+  GST_VAAPI_DISPLAY_UNLOCK (display);
   if (!vaapi_check_status (status, "vaQuerySurfaceStatus()"))
     return FALSE;
 
@@ -1026,8 +999,6 @@ gst_vaapi_surface_query_status (GstVaapiSurface * surface,
  * gst_vaapi_surface_set_subpictures_from_composition:
  * @surface: a #GstVaapiSurface
  * @compostion: a #GstVideoOverlayCompositon
- * @propagate_context: a flag specifying whether to apply composition
- *     to the parent context, if any
  *
  * Helper to update the subpictures from #GstVideoOverlayCompositon. Sending
  * a NULL composition will clear all the current subpictures. Note that this
@@ -1037,18 +1008,14 @@ gst_vaapi_surface_query_status (GstVaapiSurface * surface,
  */
 gboolean
 gst_vaapi_surface_set_subpictures_from_composition (GstVaapiSurface * surface,
-    GstVideoOverlayComposition * composition, gboolean propagate_context)
+    GstVideoOverlayComposition * composition)
 {
   GstVaapiDisplay *display;
   guint n, nb_rectangles;
 
   g_return_val_if_fail (surface != NULL, FALSE);
 
-  if (propagate_context && surface->parent_context)
-    return gst_vaapi_context_apply_composition (surface->parent_context,
-        composition);
-
-  display = GST_VAAPI_OBJECT_DISPLAY (surface);
+  display = GST_VAAPI_SURFACE_DISPLAY (surface);
   if (!display)
     return FALSE;
 
@@ -1069,22 +1036,26 @@ gst_vaapi_surface_set_subpictures_from_composition (GstVaapiSurface * surface,
     rect = gst_video_overlay_composition_get_rectangle (composition, n);
     subpicture = gst_vaapi_subpicture_new_from_overlay_rectangle (display,
         rect);
+    if (subpicture == NULL) {
+      GST_WARNING ("could not create subpicture for rectangle %p", rect);
+      return FALSE;
+    }
 
     gst_video_overlay_rectangle_get_render_rectangle (rect,
         (gint *) & sub_rect.x, (gint *) & sub_rect.y,
         &sub_rect.width, &sub_rect.height);
 
     /* ensure that the overlay is not bigger than the surface */
-    sub_rect.y = MIN (sub_rect.y, surface->height);
-    sub_rect.width = MIN (sub_rect.width, surface->width);
+    sub_rect.y = MIN (sub_rect.y, GST_VAAPI_SURFACE_HEIGHT (surface));
+    sub_rect.width = MIN (sub_rect.width, GST_VAAPI_SURFACE_WIDTH (surface));
 
     if (!gst_vaapi_surface_associate_subpicture (surface, subpicture,
             NULL, &sub_rect)) {
       GST_WARNING ("could not render overlay rectangle %p", rect);
-      gst_vaapi_object_unref (subpicture);
+      gst_vaapi_subpicture_unref (subpicture);
       return FALSE;
     }
-    gst_vaapi_object_unref (subpicture);
+    gst_vaapi_subpicture_unref (subpicture);
   }
   return TRUE;
 }
