@@ -30,7 +30,6 @@
 #include "sysdeps.h"
 #include "gstvaapivideopool.h"
 #include "gstvaapivideopool_priv.h"
-#include "gstvaapiobject.h"
 
 #define DEBUG 1
 #include "gstvaapidebug.h"
@@ -67,8 +66,8 @@ gst_vaapi_video_pool_init (GstVaapiVideoPool * pool, GstVaapiDisplay * display,
 void
 gst_vaapi_video_pool_finalize (GstVaapiVideoPool * pool)
 {
-  g_list_free_full (pool->used_objects, gst_vaapi_object_unref);
-  g_queue_foreach (&pool->free_objects, (GFunc) gst_vaapi_object_unref, NULL);
+  g_list_free_full (pool->used_objects, (GDestroyNotify) gst_mini_object_unref);
+  g_queue_foreach (&pool->free_objects, (GFunc) gst_mini_object_unref, NULL);
   g_queue_clear (&pool->free_objects);
   gst_vaapi_display_replace (&pool->display, NULL);
   g_mutex_clear (&pool->mutex);
@@ -179,11 +178,18 @@ gst_vaapi_video_pool_get_object_unlocked (GstVaapiVideoPool * pool)
     g_mutex_lock (&pool->mutex);
     if (!object)
       return NULL;
+
+    /* Others already allocated a new one before us during we
+       release the mutex */
+    if (pool->capacity && pool->used_count >= pool->capacity) {
+      gst_mini_object_unref (object);
+      return NULL;
+    }
   }
 
   ++pool->used_count;
   pool->used_objects = g_list_prepend (pool->used_objects, object);
-  return gst_vaapi_object_ref (object);
+  return gst_mini_object_ref (object);
 }
 
 gpointer
@@ -219,7 +225,7 @@ gst_vaapi_video_pool_put_object_unlocked (GstVaapiVideoPool * pool,
   if (!elem)
     return;
 
-  gst_vaapi_object_unref (object);
+  gst_mini_object_unref (object);
   --pool->used_count;
   pool->used_objects = g_list_delete_link (pool->used_objects, elem);
   g_queue_push_tail (&pool->free_objects, object);
@@ -251,7 +257,7 @@ static inline gboolean
 gst_vaapi_video_pool_add_object_unlocked (GstVaapiVideoPool * pool,
     gpointer object)
 {
-  g_queue_push_tail (&pool->free_objects, gst_vaapi_object_ref (object));
+  g_queue_push_tail (&pool->free_objects, gst_mini_object_ref (object));
   return TRUE;
 }
 
@@ -328,30 +334,16 @@ gst_vaapi_video_pool_get_size (GstVaapiVideoPool * pool)
   return size;
 }
 
-/**
- * gst_vaapi_video_pool_reserve:
- * @pool: a #GstVaapiVideoPool
- * @n: the number of objects to pre-allocate
- *
- * Pre-allocates up to @n objects in the pool. If @n is less than or
- * equal to the number of free and used objects in the pool, this call
- * has no effect. Otherwise, it is a request for allocation of
- * additional objects.
- *
- * Return value: %TRUE on success
- */
 static gboolean
 gst_vaapi_video_pool_reserve_unlocked (GstVaapiVideoPool * pool, guint n)
 {
   guint i, num_allocated;
 
-  num_allocated = gst_vaapi_video_pool_get_size (pool) + pool->used_count;
-  if (n < num_allocated)
+  num_allocated = g_queue_get_length (&pool->free_objects) + pool->used_count;
+  if (n <= num_allocated)
     return TRUE;
 
-  if ((n -= num_allocated) > pool->capacity)
-    n = pool->capacity;
-
+  n = MIN (n, pool->capacity);
   for (i = num_allocated; i < n; i++) {
     gpointer object;
 
@@ -365,6 +357,18 @@ gst_vaapi_video_pool_reserve_unlocked (GstVaapiVideoPool * pool, guint n)
   return TRUE;
 }
 
+/**
+ * gst_vaapi_video_pool_reserve:
+ * @pool: a #GstVaapiVideoPool
+ * @n: the number of objects to pre-allocate
+ *
+ * Pre-allocates up to @n objects in the pool. If @n is less than or
+ * equal to the number of free and used objects in the pool, this call
+ * has no effect. Otherwise, it is a request for allocation of
+ * additional objects.
+ *
+ * Return value: %TRUE on success
+ */
 gboolean
 gst_vaapi_video_pool_reserve (GstVaapiVideoPool * pool, guint n)
 {
