@@ -29,7 +29,6 @@
 #include <gst/vaapi/gstvaapitypes.h>
 #include <gst/vaapi/gstvaapidisplay.h>
 #include <gst/vaapi/gstvaapisurface.h>
-#include <gst/vaapi/gstvaapipixmap.h>
 
 G_BEGIN_DECLS
 
@@ -86,15 +85,15 @@ gst_vaapi_window_set_height (GstVaapiWindow * window, guint height);
 void
 gst_vaapi_window_set_size (GstVaapiWindow * window, guint width, guint height);
 
+void
+gst_vaapi_window_set_render_rectangle (GstVaapiWindow * window, gint x, gint y,
+    gint width, gint height);
+
 gboolean
 gst_vaapi_window_put_surface (GstVaapiWindow * window,
     GstVaapiSurface * surface, const GstVaapiRectangle * src_rect,
     const GstVaapiRectangle * dst_rect, guint flags);
 
-gboolean
-gst_vaapi_window_put_pixmap (GstVaapiWindow * window, GstVaapiPixmap * pixmap,
-    const GstVaapiRectangle * src_rect, const GstVaapiRectangle * dst_rect);
-
 void
 gst_vaapi_window_reconfigure (GstVaapiWindow * window);
 
@@ -104,9 +103,7 @@ gst_vaapi_window_unblock (GstVaapiWindow * window);
 gboolean
 gst_vaapi_window_unblock_cancel (GstVaapiWindow * window);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstVaapiWindow, gst_object_unref)
-#endif
 
 G_END_DECLS
 
