@@ -83,6 +83,8 @@ struct _GstVaapiWindow
   guint check_geometry:1;
 
   /* for conversion */
+  GstVideoFormat surface_pool_format;
+  guint surface_pool_flags;
   GstVaapiVideoPool *surface_pool;
   GstVaapiFilter *filter;
   gboolean has_vpp;
@@ -123,12 +125,11 @@ struct _GstVaapiWindowClass
   gboolean (*render) (GstVaapiWindow * window, GstVaapiSurface * surface,
       const GstVaapiRectangle * src_rect, const GstVaapiRectangle * dst_rect,
       guint flags);
-  gboolean (*render_pixmap) (GstVaapiWindow * window, GstVaapiPixmap * pixmap,
-      const GstVaapiRectangle * src_rect, const GstVaapiRectangle * dst_rect);
   guintptr (*get_visual_id) (GstVaapiWindow * window);
   guintptr (*get_colormap) (GstVaapiWindow * window);
   gboolean (*unblock) (GstVaapiWindow * window);
   gboolean (*unblock_cancel) (GstVaapiWindow * window);
+  void (*set_render_rect) (GstVaapiWindow * window, gint x, gint y, gint width, gint height);
 };
 
 GstVaapiWindow *
@@ -140,6 +141,10 @@ gst_vaapi_window_vpp_convert_internal (GstVaapiWindow * window,
     GstVaapiSurface * surface, const GstVaapiRectangle * src_rect,
     const GstVaapiRectangle * dst_rect, guint flags);
 
+void
+gst_vaapi_window_set_vpp_format_internal (GstVaapiWindow * window,
+    GstVideoFormat format, guint flags);
+
 G_END_DECLS
 
 #endif /* GST_VAAPI_WINDOW_PRIV_H */
